/**
 * Objet3D applet de reprsentation d'objets en 3D
 * Copyright (C) 1998 Mickal Guessant
 *
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
 * selon les termes de la Licence Publique Gnrale GNU publie par la 
 * Free Software Foundation (version 2 ou bien toute autre version ultrieure
 * choisie par vous).
 * 
 * Ce programme est distribu car potentiellement utile, mais SANS AUCUNE 
 * GARANTIE, ni explicite ni implicite, y compris les garanties de 
 * commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
 *  la Licence Publique Gnrale GNU pour plus de dtails.
 *
 * Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en 
 * mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
 * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 * MA 02111-1307, tats-Unis.
 *
 * @author Mickal Guessant guessant@multimania.com
 *
 * Facette d'un objet
 */

import java.io.*;
import java.awt.*;

class Facette {
  private int couleur;
  private int indices[]; // indices des sommets de la facette
  private int facetteX[];//utilise par fillPolygon
  private int facetteY[];


  /** le flux d'entre doit contenir la couleur,
   *  suivie du nombre de points, suivi des indices
   *  des points */
  Facette(DataInputStream dis) throws IOException {
    couleur = dis.readInt();
    indices = new int[dis.readInt()];
    

    facetteX = new int[indices.length + 1];
    facetteY = new int[indices.length + 1];

    for (int i = 0; i < indices.length; i++)
      indices[i] = dis.readInt();
  }

  /** calcul de la distance en z de la facette,
   *  necessite le tableau des points image*/
  double distance(double image[][]) {
    double aux = 0;

    for(int i = 0; i < indices.length; i++)
      aux += image[indices[i]][2];

    return aux / indices.length;
  }

  /** attenuation de la couleur */
  private Color attenuation(int couleur, double eclairement) {
    double coef = eclairement * 0.6 + 0.4;
    if (coef > 1) coef = 1;

    // on recupere les trois couleurs
    int j = couleur & 255;
    int k = (couleur >> 8) & 255;
    int l = (couleur >> 16) & 255;
    // on applique l'attenuation
    j = (int)(coef * j);
    k = (int)(coef * k);
    l = (int)(coef * l);

    return new Color(l, k, j);
  }

  /** dessin de la surface */
  void paint(Graphics g, double image[][], double eclairement) {
    for(int i = 0; i < indices.length; i++) {
      facetteX[i] = (int)image[indices[i]][0];
      facetteY[i] = (int)image[indices[i]][1];
    }
  
    facetteX[indices.length] = facetteX[0];
    facetteY[indices.length] = facetteY[0];
  
    g.setColor(attenuation(couleur, eclairement));
    g.fillPolygon(facetteX, facetteY, facetteX.length);
  }

}


