/**
 * Objet3D applet de reprsentation d'objets en 3D
 * Copyright (C) 1998 Mickal Guessant
 *
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
 * selon les termes de la Licence Publique Gnrale GNU publie par la 
 * Free Software Foundation (version 2 ou bien toute autre version ultrieure
 * choisie par vous).
 * 
 * Ce programme est distribu car potentiellement utile, mais SANS AUCUNE 
 * GARANTIE, ni explicite ni implicite, y compris les garanties de 
 * commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
 *  la Licence Publique Gnrale GNU pour plus de dtails.
 *
 * Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en 
 * mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
 * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 * MA 02111-1307, tats-Unis.
 *
 * @author Mickal Guessant guessant@multimania.com
 *
 * Genere icosaedre.dat
 */

import java.io.*;
import java.awt.Color;

class Icosaedre {
  static double sommets[][] = new double[12][3];

  static int facettes[][] = {{(int)(Math.random() * 16777215), 0, 1, 2},
                             {(int)(Math.random() * 16777215), 0, 2, 3},
                             {(int)(Math.random() * 16777215), 0, 3, 4},
                             {(int)(Math.random() * 16777215), 0, 4, 5},
                             {(int)(Math.random() * 16777215), 0, 5, 1},
                  
                             {(int)(Math.random() * 16777215), 1, 6, 2},
                             {(int)(Math.random() * 16777215), 2, 7, 3},
                             {(int)(Math.random() * 16777215), 3, 8, 4},
                             {(int)(Math.random() * 16777215), 4, 9, 5},
                             {(int)(Math.random() * 16777215), 5, 10, 1},

                             {(int)(Math.random() * 16777215), 2, 6, 7},
                             {(int)(Math.random() * 16777215), 3, 7, 8},
                             {(int)(Math.random() * 16777215), 4, 8, 9},
                             {(int)(Math.random() * 16777215), 5, 9, 10},
                             {(int)(Math.random() * 16777215), 1, 10, 6},
                       
                             {(int)(Math.random() * 16777215), 6, 11, 7},
                             {(int)(Math.random() * 16777215), 7, 11, 8},
                             {(int)(Math.random() * 16777215), 8, 11, 9},
                             {(int)(Math.random() * 16777215), 9, 11, 10},
                             {(int)(Math.random() * 16777215), 10, 11, 6},
                            };

  public static void main(String argv[]) {
    int blanc = Color.white.getRGB();                             

    // initialisation des sommets
    double r = 1;
    double phi = 1.10714871781;
    double piCinq = Math.PI / 5;

    spherique(0, r, 0, 0);
    for (int i = 1; i < 6; i++) {
      spherique(i, r, (i - 1) * 2 * piCinq, phi);
      spherique(i + 5, r, (i - 1) * 2 * piCinq + piCinq, Math.PI - phi);
    }
    spherique(11, r, 0, Math.PI);

    try {
      // ouverture du fichier
      DataOutputStream dos = new DataOutputStream(
        new FileOutputStream("icosaedre.dat"));

      dos.writeUTF("Objet : Icosaedre");

      dos.writeInt(sommets.length);

      for (int i = 0; i < sommets.length; i++) {
        dos.writeDouble(sommets[i][0]);
        dos.writeDouble(sommets[i][1]);
        dos.writeDouble(sommets[i][2]);
      }

      dos.writeInt(facettes.length);

      for (int i = 0; i < facettes.length; i++) {
        dos.writeInt(facettes[i][0]);
        dos.writeInt(3);
        dos.writeInt(facettes[i][1]);
        dos.writeInt(facettes[i][2]);
        dos.writeInt(facettes[i][3]);
      }

      double normale[];

      for (int i = 0; i < facettes.length; i++) {
        normale = getNormale(i);

        dos.writeDouble(normale[0]);
        dos.writeDouble(normale[1]);
        dos.writeDouble(normale[2]);
      }


      dos.close();
    } catch (Exception e) {
      System.out.println(e.getMessage());
    }
  }

  // fixe le sommet i donne en coordonnees spheriques
  static void spherique(int i, double r, double theta, double phi) {
    Vecteur v = new Vecteur();
    v.spherique(r, theta, phi);

    sommets[i] = v.getValue();
  }

  static double[] getNormale(int i) {
    Vecteur v1, v2, v3, normale;

    // creation des vecteurs de la facette
    v1 = new Vecteur(sommets[facettes[i][1]]);
    v2 = new Vecteur(sommets[facettes[i][2]]);
    v3 = new Vecteur(sommets[facettes[i][3]]);

    // calcul des cotes
    v2.soustrait(v1);
    v3.soustrait(v1);

    // calcul de la normale
    normale = new Vecteur(v2, v3);
    normale.normalise();

    return normale.getValue();
  }

}
