/**
 * Objet3D applet de reprsentation d'objets en 3D
 * Copyright (C) 1998 Mickal Guessant
 *
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
 * selon les termes de la Licence Publique Gnrale GNU publie par la 
 * Free Software Foundation (version 2 ou bien toute autre version ultrieure
 * choisie par vous).
 * 
 * Ce programme est distribu car potentiellement utile, mais SANS AUCUNE 
 * GARANTIE, ni explicite ni implicite, y compris les garanties de 
 * commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
 *  la Licence Publique Gnrale GNU pour plus de dtails.
 *
 * Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en 
 * mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
 * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 * MA 02111-1307, tats-Unis.
 *
 * @author Mickal Guessant guessant@multimania.com
 *
 * Convertit de iv vers dat
 */

import java.io.*;
import java.awt.Color;
import java.util.Vector;
import java.util.Enumeration;

class IvToDat {
  StreamTokenizer st;
  int type;
  String information = "";
  int couleursRef, sommetsRef, normalesRef; // sommet et normale de reference

  Vector sommets = new Vector(10, 5),
         facettes = new Vector(10, 5),
         couleurs = new Vector(10, 5),
         indexCouleurs = new Vector(10, 5),
         normales = new Vector(10, 5),
         normalIndex = new Vector(10, 5);

  void Info() throws IOException {    
    st.nextToken(); // {
    st.nextToken(); // string
    st.nextToken(); // "

    infoSyntax();

    st.nextToken();

    information = information + "\n" + st.sval;

    normalSyntax();

  }

  void infoSyntax() {
    st.resetSyntax();
    st.wordChars(0, 255);
    st.ordinaryChar('"');
  }

  void normalSyntax() {
    st.resetSyntax();
    st.whitespaceChars(0, ' ');
    st.wordChars('a', 'z');
    st.wordChars('A', 'Z');
    st.wordChars('_', '_');
    st.ordinaryChar('"');
    st.parseNumbers();
  }

  void rgb() throws IOException {
    int couleur;

    couleursRef = couleurs.size();

    while (((type = st.nextToken()) != '}') && (type != ']')) {
      if (type == StreamTokenizer.TT_NUMBER) {
        couleur = (int)(st.nval * 255);
        st.nextToken();couleur = (couleur << 8) + (int)(st.nval * 255);
        st.nextToken();couleur = (couleur << 8) + (int)(st.nval * 255);
        couleurs.addElement(new Integer(couleur));
      }
    }
  }

  void materialIndex() throws IOException {

    while (((type = st.nextToken()) != '}') && (type != ']')) {
      if (type == StreamTokenizer.TT_NUMBER) {
        indexCouleurs.addElement(new Integer((int)st.nval));
      }
    }
  }

  double parseDouble() throws IOException {
    double aux = st.nval;

    type = st.nextToken();
    if (type == StreamTokenizer.TT_WORD) {
      aux = aux * Math.pow(10,((new Double(st.sval.substring(1))).doubleValue()));
    } else st.pushBack();

    return aux;
  }

  void vector() throws IOException {
    double sommet[];    

    normalesRef = normales.size();

    while (((type = st.nextToken()) != '}') && (type != ']')) {
      if (type == StreamTokenizer.TT_NUMBER) {
        sommet = new double[3];

        sommet[0] = parseDouble();
        st.nextToken();sommet[1] = parseDouble();
        st.nextToken();sommet[2] = parseDouble();

        normales.addElement(sommet);
      }
    }
  }
  
  void Coordinate3() throws IOException {
    double sommet[];

    sommetsRef = sommets.size();

    while (((type = st.nextToken()) != '}') && (type != ']')) {
      if (type == StreamTokenizer.TT_NUMBER) {
        sommet = new double[3];

        sommet[0] = parseDouble();
        st.nextToken();sommet[1] = parseDouble();
        st.nextToken();sommet[2] = parseDouble();

        sommets.addElement(sommet);
      }
    }
  }

  void coordIndex() throws IOException {
    int facette[] = new int[200], aux[];
    int indice = 0;

    while (((type = st.nextToken()) != '}') && (type != ']')) {
      if (type == StreamTokenizer.TT_NUMBER) {
        
        if (st.nval == -1) {
          aux = new int[indice + 1]; 
          indice = 0;

          aux[0] = Color.white.getRGB(); //(int)(Math.random() * 16777215);

          for (int i = 1; i < aux.length; i++)
            aux[i] = sommetsRef + facette[i - 1];

          facettes.addElement(aux);

        } else facette[indice++] = (int)st.nval;
      }
    }
  }

  void normalIndex() throws IOException {
    int facette[] = new int[200], aux[];
    int indice = 0;

    while (((type = st.nextToken()) != '}') && (type != ']')) {
      if (type == StreamTokenizer.TT_NUMBER) {
        
        if (st.nval == -1) {
          aux = new int[indice]; 
          indice = 0;

          for (int i = 0; i < aux.length; i++)
            aux[i] = normalesRef + facette[i];

          normalIndex.addElement(aux);

        } else facette[indice++] = (int)st.nval;
      }
    }
  }

  void numVertices() throws IOException {
    int facette[];
    int indice = 0;

    while (((type = st.nextToken()) != '}') && (type != ']')) {
      if (type == StreamTokenizer.TT_NUMBER) {
        facette = new int[(int)st.nval + 1];

        facette[0] = Color.white.getRGB(); //(int)(Math.random() * 16777215);
        for(int i = 1; i < facette.length; i++)
          facette[i] = indice++;

          facettes.addElement(facette);
      }
    }
  }

  void writeSommets(DataOutputStream dos) throws IOException {
    double sommet[];

    Enumeration e = sommets.elements();

    dos.writeInt(sommets.size());
    System.out.print(sommets.size());
    System.out.println(" sommets");


    while (e.hasMoreElements()) {
      sommet = (double[])e.nextElement();

      for (int i = 0; i < sommet.length; i++) 
        dos.writeDouble(sommet[i]);
    }
  }

  double[] getNormale(int[] facette) {
    Vecteur v1, v2, v3, normale;

    // creation des vecteurs de la facette
    v1 = new Vecteur((double[])sommets.elementAt(facette[1]));
    v2 = new Vecteur((double[])sommets.elementAt(facette[2]));
    v3 = new Vecteur((double[])sommets.elementAt(facette[3]));

    // calcul des cotes
    v2.soustrait(v1);
    v3.soustrait(v1);

    // calcul de la normale
    normale = new Vecteur(v2, v3);
    normale.normalise();

    return normale.getValue();
  }
      
  void writeNormales(DataOutputStream dos) throws IOException {
    int index[];
    double aux[];
    Vecteur normale;

    Enumeration e = normalIndex.elements();

    System.out.print(normalIndex.size());
    System.out.println(" normales");

    if (normalIndex.size() > 0)
      while (e.hasMoreElements()) {
        index = (int[])e.nextElement();
  
        normale = new Vecteur((double[])normales.elementAt(index[0]));
        for(int i = 1; i < index.length; i++)
          normale.ajoute(new Vecteur((double[])normales.elementAt(index[i])));
        normale.normalise();

        aux = normale.getValue();
        for (int i = 0; i < 3; i++) 
          dos.writeDouble(aux[i]);
      }
    else {
      System.out.println("Calcul des normales");

      e = facettes.elements();
      while (e.hasMoreElements()) {
        aux = getNormale((int[])e.nextElement());
        for (int i = 0; i < 3; i++) 
          dos.writeDouble(aux[i]);
      }
    }
  }

  void writeFacettes(DataOutputStream dos) throws IOException {
    int couleur = 0;
    int facette[], indexCouleur;
    boolean couleursDefinies = true;

    Enumeration e = facettes.elements(),
                enumCouleurs = indexCouleurs.elements();
                
    if (indexCouleurs.size() == 0) {
      couleursDefinies = false;
      if (couleurs.size() == 0)
        couleur = Color.white.getRGB();
      else couleur = ((Integer)couleurs.elementAt(couleursRef)).intValue();
    }

    dos.writeInt(facettes.size());
    System.out.print(facettes.size());
    System.out.println(" facettes");

    while (e.hasMoreElements()) {
      facette = (int[])e.nextElement();

      if (couleursDefinies) {
        indexCouleur = couleursRef + ((Integer)enumCouleurs.nextElement()).intValue();
        dos.writeInt(((Integer)couleurs.elementAt(indexCouleur)).intValue());

      } else dos.writeInt(couleur);
      
      dos.writeInt(facette.length - 1);

      for (int i = 1; i < facette.length; i++) 
        dos.writeInt(facette[i]);
    }
  }


  /** constructeur, necessite le nom du fichier */
  public IvToDat(String nomFichier) {
    try {
      // ouverture du fichier
      BufferedInputStream bis = new BufferedInputStream(
        new FileInputStream(nomFichier + ".iv"));
        
      st = new StreamTokenizer(bis);
      normalSyntax();
      
      while ((type = st.nextToken()) != StreamTokenizer.TT_EOF) {
	if (type == StreamTokenizer.TT_WORD) {
          // System.out.println(st.sval);
          if (st.sval.equals("Coordinate3"))
            Coordinate3();
          else if (st.sval.equals("numVertices"))
            numVertices();
          else if (st.sval.equals("coordIndex"))
            coordIndex();
          else if (st.sval.equals("rgb") || st.sval.equals("Material"))
            rgb();
          else if (st.sval.equals("materialIndex"))
            materialIndex();
          else if (st.sval.equals("vector"))
            vector();
          else if (st.sval.equals("normalIndex"))
            normalIndex();
          else if (st.sval.equals("Info"))
            Info();
          else if (!(st.sval.equals("Inventor") ||
                     st.sval.equals("V1.0") ||  
                     st.sval.equals("ascii") ||  
                     st.sval.equals("ORDERED") ||  
                     st.sval.equals("CONVEX") ||  
                     st.sval.equals("Separator") ||  
                     st.sval.equals("ShapeHints") ||  
                     st.sval.equals("hints") ||  
                     st.sval.equals("SOLID") ||  
                     st.sval.equals("Normal") ||  
                     st.sval.equals("NormalBinding") ||  
                     st.sval.equals("value") ||  
                     st.sval.equals("PER_VERTEX_INDEXED") ||  
                     st.sval.equals("IndexedFaceSet") ||  
                     st.sval.equals("V1.0") 
                    ))
            System.out.println("Non reconnu : " + st.sval);
        }
      }
      
      bis.close();
        
      DataOutputStream dos = new DataOutputStream(
        new BufferedOutputStream(
        new FileOutputStream(nomFichier + ".dat")));

      dos.writeUTF(information);
      writeSommets(dos);
      writeFacettes(dos);
      writeNormales(dos);

      dos.close();

    } catch (Exception e) {
      System.out.println(e.getMessage());
      e.printStackTrace();
    }

  }

  public static void main(String argv[]) {
    if (argv.length != 1) {
      System.out.println("Usage : IvToDat nom");
      System.exit(1);
    }

    IvToDat ivtdt = new IvToDat(argv[0]);

  }

}






