/**
 * Objet3D applet de reprsentation d'objets en 3D
 * Copyright (C) 1998 Mickal Guessant
 *
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
 * selon les termes de la Licence Publique Gnrale GNU publie par la 
 * Free Software Foundation (version 2 ou bien toute autre version ultrieure
 * choisie par vous).
 * 
 * Ce programme est distribu car potentiellement utile, mais SANS AUCUNE 
 * GARANTIE, ni explicite ni implicite, y compris les garanties de 
 * commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
 *  la Licence Publique Gnrale GNU pour plus de dtails.
 *
 * Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en 
 * mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
 * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 * MA 02111-1307, tats-Unis.
 *
 * @author Mickal Guessant guessant@multimania.com
 *
 * Objet 3D, pouvant etre manipule, affiche...
 */

import java.awt.Color;
import java.awt.Graphics;
import java.io.*;

class Objet3D {
  private double sommets[][]; // tableau des sommets de l'objet
  private double image[][]; // tableau des sommets apres transformation

  private Facette facettes[]; // tableau des facettes

  private double normales[][]; // tableau des normales aux facettes
  private double eclairement[]; // tableau de l'eclairement des facettes

  private int largeur, hauteur; // taille de la fenetre

  private int index[]; // tableau d'indice pour distanceZ
  private int nbFacettesVisibles; // nombre de facettes visibles
  private double distanceZ[]; // distance en z de chaque facette

  private double mt[][] = new double[4][4]; // matrice de transformation

  private String information; // infos sur l'objet

  /** constructeur, necessite la largeur et la hauteur de l'image,
   * ainsi qu'un flux contenant la liste des sommets et des facettes */
  Objet3D(int largeur, int hauteur, DataInputStream dis) 
    throws IOException {

    this.largeur = largeur;
    this.hauteur = hauteur;

    information = dis.readUTF();

    // lecture de la liste des sommets
    sommets = new double[dis.readInt()][3];

    for (int i = 0 ; i < sommets.length ; i++) {
      sommets[i][0] = dis.readDouble();
      sommets[i][1] = dis.readDouble();
      sommets[i][2] = dis.readDouble();
    }

    normaliseObjet();

    // lecture des facettes
    facettes = new Facette[dis.readInt()];

    for (int i = 0 ; i < facettes.length ; i++)
      facettes[i] = new Facette(dis);

    normales = new double[facettes.length][3];
    for (int i = 0 ; i < normales.length ; i++) {
      normales[i][0] = dis.readDouble();
      normales[i][1] = dis.readDouble();
      normales[i][2] = dis.readDouble();
    }

    eclairement = new double[facettes.length];

    // creation du tableau image
    image = new double[sommets.length][3];

    distanceZ = new double[facettes.length];
    index = new int[facettes.length];

    for(int i = facettes.length - 1; i >= 0 ; i--) index[i] = i;
  }

  /** informations sur l'objet */
  String getInformation() {
    return information;
  }

  /** normalise les dimensions de l'objet*/
  private void normaliseObjet() {
  double min[] = {Double.MAX_VALUE, Double.MAX_VALUE, Double.MAX_VALUE},
         max[] = {Double.MIN_VALUE, Double.MIN_VALUE, Double.MIN_VALUE};

  // calcul des extrema
  for (int i = 0; i < sommets.length; i++) {
    for (int j = 0; j < 3; j++) {
      if (sommets[i][j] < min[j]) min[j] = sommets[i][j];
      if (sommets[i][j] > max[j]) max[j] = sommets[i][j];
    }
  }

  // calcul des intervalles de valeurs sur chaque dimension
  double intervalle[] = new double[3];
  for (int j = 0; j < 3; j++) intervalle[j] = max[j] - min[j];

  // calcul des milieux
  double milieu[] = new double[3];
  for (int j = 0; j < 3; j++) milieu[j] = (max[j] + min[j]) / 2.0;

  // determination de l'intervalle maximum
  double maxIntervalle = intervalle[0];
  if (intervalle[1] > maxIntervalle) maxIntervalle = intervalle[1];
  if (intervalle[2] > maxIntervalle) maxIntervalle = intervalle[2];

  for (int i = 0; i < sommets.length; i++)
    for (int j = 0; j < 3; j++)
      sommets[i][j] = (sommets[i][j] - milieu[j]) / maxIntervalle; 

  }

  /** applique la transformation aux sommets vers image*/
  private void transformation() {
    double x, y, z, aux;

    nbFacettesVisibles = 0;

    for (int i = 0; i < eclairement.length; i++) {
      x = normales[i][0];
      y = normales[i][1];
      z = normales[i][2];

      // on applique la matrice de transformation au point
            
      aux = x * mt[0][3] + y * mt[1][3] + z * mt[2][3];

      eclairement[i] = - aux /
        (x * mt[0][2] + y * mt[1][2] + z * mt[2][2] + mt[3][2]);

      if (eclairement[i] > 0) index[nbFacettesVisibles++] = i;
    }

    for (int i = sommets.length - 1; i >= 0; i--) {
      x = sommets[i][0];
      y = sommets[i][1];
      z = sommets[i][2];

      // on applique la matrice de transformation au point
            
      aux = x * mt[0][3] + y * mt[1][3] + z * mt[2][3] + mt[3][3];

      image[i][0] = (x * mt[0][0] + y * mt[1][0] + z * mt[2][0] + mt[3][0])
                    / aux;
      image[i][1] = (x * mt[0][1] + y * mt[1][1] + z * mt[2][1] + mt[3][1])
                    / aux;
      // pour z, on recupere la valeur precedant la perspective
      image[i][2] = aux /
                    (x * mt[0][2] + y * mt[1][2] + z * mt[2][2] + mt[3][2]);                    
    }
  }


  /** dessine la surface sur g */
  void paint(Graphics g) {

    // applique la transformation de sommets vers image
    transformation();

    // calcule les distances en z de chaque facette visible
    for (int i = 0; i < nbFacettesVisibles; i++)
      distanceZ[index[i]] = facettes[index[i]].distance(image);

    // tri du tableau des distances
    triRapide(0, nbFacettesVisibles - 1);

    // dessin de la surface
    // (de chaque facette par ordre decroissant en z)
    for (int i = nbFacettesVisibles -1; i >= 0 ; i--)
      facettes[index[i]].paint(g, image, eclairement[index[i]]);
  }


  /** calcule la matrice de transformation pour la perspective */
  void perspective(double focale) {
    initMat(mt); // mt = identite

    mt[0][0] = focale * largeur;
    mt[1][1] = - focale * largeur;
    mt[2][2] = 0;
    mt[3][2] = 1;
    mt[2][3] = 1;
    mt[3][3] = 0;
    mt[2][0] = largeur / 2.0;
    mt[2][1] = hauteur / 2.0;
  }

  /** applique la translation a mt */
  void translation(double x, double y, double z) {
    mt[3][0] += x * mt[0][0] + y * mt[1][0] + z * mt[2][0];
    mt[3][1] += x * mt[0][1] + y * mt[1][1] + z * mt[2][1];
    mt[3][2] += x * mt[0][2] + y * mt[1][2] + z * mt[2][2];
    mt[3][3] += x * mt[0][3] + y * mt[1][3] + z * mt[2][3];
  }

  /** applique la rotation en x a mt */
  void rotationX(double angle) {
    double aux[] = new double[4];

    double cosAngle = Math.cos(angle);
    double sinAngle = Math.sin(angle);

    for (int i = 3; i >= 0 ; i--)
      aux[i] = cosAngle * mt[1][i] + sinAngle * mt[2][i];
    
    for (int i = 3; i >=0 ; i--) {
      mt[2][i] = -sinAngle * mt[1][i] + cosAngle * mt[2][i];
      mt[1][i] = aux[i];
    }
  }

  /** applique la rotation en y a mt */
  void rotationY(double angle) {
    double aux[] = new double[4];

    double cosAngle = Math.cos(angle);
    double sinAngle = Math.sin(angle);

    for (int i = 3; i >= 0 ; i--)
      aux[i] = cosAngle * mt[0][i] - sinAngle * mt[2][i];
    
    for (int i = 3; i >= 0 ; i--) {
      mt[2][i] = sinAngle * mt[0][i] + cosAngle * mt[2][i];
      mt[0][i] = aux[i];
    }
  }

  /** applique la rotation en z a mt */
  void rotationZ(double angle) {
    double aux[] = new double[4];

    double cosAngle = Math.cos(angle);
    double sinAngle = Math.sin(angle);

    for (int i = 3; i >= 0 ; i--)
      aux[i] = cosAngle * mt[0][i] + sinAngle * mt[1][i];
    
    for (int i = 3; i >= 0 ; i--) {
      mt[1][i] = -sinAngle * mt[0][i] + cosAngle * mt[1][i];
      mt[0][i] = aux[i];
    }
  }

  /** initialise la matrice 4x4 a l'identite */
  private void initMat(double mat[][]) {
    mat[0][0] = 1.0;
    mat[0][1] = 0.0;
    mat[0][2] = 0.0;
    mat[0][3] = 0.0;

    mat[1][0] = 0.0;
    mat[1][1] = 1.0;
    mat[1][2] = 0.0;
    mat[1][3] = 0.0;

    mat[2][0] = 0.0;
    mat[2][1] = 0.0;
    mat[2][2] = 1.0;
    mat[2][3] = 0.0;

    mat[3][0] = 0.0;
    mat[3][1] = 0.0;
    mat[3][2] = 0.0;
    mat[3][3] = 1.0;
  }


  /** tri bulle du tableau distanceZ indice par index */
  private void triBulle(int debut, int fin) {
    int aux;
    boolean trie;

    for (int i = fin; i > 1; i--) {
	trie = true;
	
	for (int k = debut; k < i; k++)
	  if (distanceZ[index[k]] > distanceZ[index[k + 1]])
	    {
	      aux = index[k];
	      index[k] = index[k + 1];
	      index[k + 1] = aux;
	      trie = false;
	    }
	if (trie) return;
      }
  }


  /** tri rapide du tableau distanceZ indice par index */
  private void triRapide(int debut, int fin) {
    int i, j, aux;
    double pivot;

    if ( fin > debut) {

      // si le tableau est petit, le tri bulle est plus efficace
      if (fin < debut + 5) {
	triBulle(debut, fin);

      } else {
	// indices locaux
	i = debut;
	j = fin;
    
	pivot = distanceZ[index[(debut + fin) / 2]];
	
	while( i <= j )
	  {
	    while( ( i < fin ) && (distanceZ[index[i]] < pivot)) i++;
	    while( ( j > debut ) && (distanceZ[index[j]] > pivot)) j--;
	    if( i <= j )
	      {
		aux = index[j];
		index[j] = index[i];
		index[i] = aux;
		
		i++;
		j--;
	      }
	  }

	triRapide(debut, j);
	triRapide(i, fin);
      }
    }
    
  }

}


