/**
 * Objet3D applet de reprsentation d'objets en 3D
 * Copyright (C) 1998 Mickal Guessant
 *
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
 * selon les termes de la Licence Publique Gnrale GNU publie par la 
 * Free Software Foundation (version 2 ou bien toute autre version ultrieure
 * choisie par vous).
 * 
 * Ce programme est distribu car potentiellement utile, mais SANS AUCUNE 
 * GARANTIE, ni explicite ni implicite, y compris les garanties de 
 * commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
 *  la Licence Publique Gnrale GNU pour plus de dtails.
 *
 * Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en 
 * mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
 * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 * MA 02111-1307, tats-Unis.
 *
 * @author Mickal Guessant guessant@multimania.com
 *
 * Applet de representation d'objets 3D
 */

import java.applet.Applet;
import java.net.MalformedURLException;
import java.net.URL;
import java.awt.*;
import java.io.*;

public class Objet3DApplet extends Applet 
  implements Runnable, ControleObjet3D {
  ObjetCanvas objetCanvas;
  Thread thread;
  TextField rotationX, rotationY, zoom;
  Button aPropos;
  TextArea info;

  public void init () {
    thread = new Thread(this);
    thread.start();
  }

  public void run() {

    // creation des boutons et des champs texte
    Panel p = new Panel();

    aPropos = new Button("A propos");
    p.add(aPropos);

    p.add(new Label("rot X"));
    rotationX = new TextField(5);
    p.add(rotationX);

    p.add(new Label("rot Y"));
    rotationY = new TextField(5);
    p.add(rotationY);

    p.add(new Label("zoom"));
    zoom = new TextField(4);
    p.add(zoom);

    add(p);
    validate();

    // panel pour les messages de chargement
    p = new Panel();
    p.setLayout(new GridLayout(0,1));
    add(p);

    try {
      p.add(new Label("Chargement de " + getParameter("fichierObjet")));
      validate();

      // chargement du fichier de valeurs precises
      DataInputStream donneesObjet = new 
        DataInputStream(new URL(getDocumentBase(), 
          getParameter("fichierObjet")).openStream());

      // creation de l'objet
      objetCanvas = new ObjetCanvas(size().width,
			            size().height - p.size().height,
                                    donneesObjet, this);
                                    
      donneesObjet.close();

      info = new TextArea("Objet3D applet de reprsentation d'objets en 3D\n" +
													"Copyright (C) 1998 Mickal Guessant\n\n" + 
                          objetCanvas.getInformation(), 12, 45);
      info.resize(objetCanvas.size());
                                   
      setVue(objetCanvas.getRotX(),
	     objetCanvas.getRotY(),
	     objetCanvas.getZoom());

      remove(p);
      add(objetCanvas);

      showStatus("Execution de Objet 3D");

      validate();

    } catch (MalformedURLException e) {
      p.add(new Label(("Objet 3D : erreur de fichier " + e.getMessage())));

    } catch (NumberFormatException e) {
      p.add(new Label(("Objet 3D : erreur de parametre " + e.getMessage())));

    } catch (IOException e) {
      p.add(new Label(("Objet 3D : erreur de lecture " + e.getMessage())));

    } catch (Exception e) {
      System.out.println(e.getMessage());
      e.printStackTrace();

    }

    validate();
  }

  /** permet de mettre a jour les TextFields */
  public void setVue(double rotX, double rotY, double zoo) {
    rotationX.setText(String.valueOf(rotX));
    rotationY.setText(String.valueOf(rotY));
    zoom.setText(String.valueOf(zoo));
  }

  public boolean action(Event event, Object arg) {
    // mise a jour de l'objet en fonction des valeurs entrees
    if ((event.target == rotationX) ||
	(event.target == rotationY) ||
        (event.target == zoom)) {

      try {
	  double rotX = (new Double(rotationX.getText())).doubleValue();
	  double rotY = (new Double(rotationY.getText())).doubleValue();
	  double zoo =  (new Double(zoom.getText())).doubleValue();
	
	  objetCanvas.setVue(rotX, rotY, zoo);

      } catch (NumberFormatException e) {
	  showStatus("Objet3DApplet : valeur invalide " + e.getMessage());
      }

      return true;      
    } else if (event.target == aPropos) {
      if (aPropos.getLabel().equals("A propos")) {
        aPropos.setLabel("Objet");
        remove(objetCanvas);        
        add(info);
      
        validate();

      } else {
        aPropos.setLabel("A propos");
        remove(info);
        add(objetCanvas);

        validate();

      }
    
      return true;
    }
      

    return false;
  }

  /** information sur les parametres */
  public String[][] getParameterInfo() {
    String[][] info = {
      {"fichierObjet",     "URL",          "fichier de donnees objet"},
    };
    return info;
  }

  /** info sur l'applet */
  public String getAppletInfo() {
    return "Objet3D par Mickael Guessant";
  }
     
}
