/**
 * Objet3D applet de reprsentation d'objets en 3D
 * Copyright (C) 1998 Mickal Guessant
 *
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
 * selon les termes de la Licence Publique Gnrale GNU publie par la 
 * Free Software Foundation (version 2 ou bien toute autre version ultrieure
 * choisie par vous).
 * 
 * Ce programme est distribu car potentiellement utile, mais SANS AUCUNE 
 * GARANTIE, ni explicite ni implicite, y compris les garanties de 
 * commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
 *  la Licence Publique Gnrale GNU pour plus de dtails.
 *
 * Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en 
 * mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
 * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 * MA 02111-1307, tats-Unis.
 *
 * @author Mickal Guessant guessant@multimania.com
 *
 * Panel de gestion d'un objet
 * le parent doit implementer ControlObjet3D
 */

import java.awt.*;
import java.io.*;

class ObjetCanvas extends Canvas {
  private int largeur, hauteur; // taille de la fenetre
  private int precx, precy; // position precedente de la souris

  private Image canvasImage; // image
  private Graphics canvasGraphics;  // Graphics associe a l'image
  private ControleObjet3D parent;

  private boolean precinit, // indicateur position precedente                                                         // initialisee,
                  controle; // touche controle enfoncee

  private Objet3D objet; // l'objet a representer                                                   // representer

  private double rotY = 0.0,
    rotX = 0.0, zoom = 1; // situation courante

  /** La construction de l'objet necessite un flux */
  ObjetCanvas(int largeur, int hauteur,
              DataInputStream donneesObjet, ControleObjet3D parent)
    throws IOException {

    this.parent = parent;
    this.largeur = largeur;
    this.hauteur = hauteur;

    resize(largeur, hauteur);

    // calcul des deux objets
    objet = new Objet3D(largeur, hauteur, donneesObjet);
  }

  /** informations sur l'objet */
  String getInformation() {
    return objet.getInformation();
  }

  void setVue(double rotX, double rotY, double zoom) {
    if (zoom < 0.01) zoom = 0.1;
    if (zoom > 3) zoom = 3;

    this.rotX = rotX;
    this.rotY = rotY;
    this.zoom = zoom;

    dessineObjet();

    parent.setVue(this.rotX, this.rotY, this.zoom);
  }

  double getRotX() { return rotX;}
  double getRotY() { return rotY;}
  double getZoom() { return zoom;}


  /** dessine l'objet */
  private void dessineObjet() {
    objet.perspective(zoom);
    objet.translation(0.0, 0.0, 2);
    objet.rotationX(rotX);
    objet.rotationY(rotY);

    // couleur de fond
    canvasGraphics.setColor(Color.black);
    canvasGraphics.fillRect(0, 0, largeur, hauteur);

    objet.paint(canvasGraphics);

    (getGraphics()).drawImage(canvasImage, 0, 0, this);
  }

  public boolean mouseDown(Event event, int i, int j) {
    precinit = false;
    controle = event.controlDown();
    return true;
  }

  public boolean mouseUp(Event event, int i, int j) {;
    dessineObjet();

    parent.setVue(rotX, rotY, zoom);

    return true;
  }

  public boolean mouseDrag(Event event, int x, int y) {
    if (precinit) {

      // calcule la modification

      rotY -= 0.005 * (x - precx);

      if (controle) {
	  zoom -= 0.002 * (y - precy);
        if (zoom < 0.01) zoom = 0.01;
        if (zoom > 3) zoom = 3;
	
      } else {
	  rotX -= 0.005 * (y - precy);
      }
            
      dessineObjet();

    } else precinit = true;

    precx = x;
    precy = y;

    return true;
  }

  public void paint(Graphics g) {
    if (canvasImage == null) {
      while (canvasImage == null)
        canvasImage = createImage(largeur, hauteur);
      canvasGraphics = canvasImage.getGraphics();

      dessineObjet();

    } else g.drawImage(canvasImage, 0, 0, this);
  }

}



