/**
 * Objet3D applet de reprsentation d'objets en 3D
 * Copyright (C) 1998 Mickal Guessant
 *
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
 * selon les termes de la Licence Publique Gnrale GNU publie par la 
 * Free Software Foundation (version 2 ou bien toute autre version ultrieure
 * choisie par vous).
 * 
 * Ce programme est distribu car potentiellement utile, mais SANS AUCUNE 
 * GARANTIE, ni explicite ni implicite, y compris les garanties de 
 * commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
 *  la Licence Publique Gnrale GNU pour plus de dtails.
 *
 * Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en 
 * mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
 * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 * MA 02111-1307, tats-Unis.
 *
 * @author Mickal Guessant guessant@multimania.com
 *
 * Vecteur 3D
 */

class Vecteur {
  double vecteur[];

  /** initialise le vecteur */
  Vecteur() {
    vecteur = new double[3];
  }

  /** construit le vecteur a partir du tableau
    * contenant les composantes */
  Vecteur(double vecteur[]) {
    this.vecteur = new double[3];

    this.vecteur[0] = vecteur[0];
    this.vecteur[1] = vecteur[1];
    this.vecteur[2] = vecteur[2];
  }

  /** construit le vecteur a partir de ses
    * composantes cartesiennes */
  Vecteur(double x, double y, double z) {
    vecteur = new double[3];
    vecteur[0] = x;
    vecteur[1] = y;
    vecteur[2] = z;
  }
 
  /** construit le vecteur produit vectoriel
    * des vecteurs passes en parametres */
  Vecteur(Vecteur v1, Vecteur v2) {
    double aux1[] = v1.getValue();
    double aux2[] = v2.getValue();

    vecteur = new double[3];
    
    vecteur[0] = aux1[1] * aux2[2] - aux1[2] * aux2[1];
    vecteur[1] = aux1[2] * aux2[0] - aux1[0] * aux2[2];
    vecteur[2] = aux1[0] * aux2[1] - aux1[1] * aux2[0];    
  }

  /** construit le vecteur a partir de coordonnees
    *  spheriques */
  void spherique(double r, double theta, double phi) {
    vecteur[0] = r * Math.cos(theta) * Math.sin(phi);
    vecteur[1] = r * Math.sin(theta) * Math.sin(phi);
    vecteur[2] = r * Math.cos(phi);
  }

  /** renvoie le tableau des composantes */
  double[] getValue() {
    return vecteur;
  }

  /** calcule la norme du vecteur */
  double norme() {
    return Math.sqrt(vecteur[0] * vecteur[0] +
                     vecteur[1] * vecteur[1] +
                     vecteur[2] * vecteur[2]);
  }

  /** normalise le vecteur */
  void normalise() {
    double norme = norme();

    vecteur[0] /= norme;
    vecteur[1] /= norme;
    vecteur[2] /= norme;
  }

  /** additionne le vecteur passe en parametre
    * au vecteur courant */
  void ajoute(Vecteur v) {
    double aux[] = v.getValue();
    vecteur[0] += aux[0];
    vecteur[1] += aux[1];
    vecteur[2] += aux[2];    
  }

  /** soustrait le vecteur passe en parametre
    * au vecteur courant */
  void soustrait(Vecteur v) {
    double aux[] = v.getValue();
    vecteur[0] -= aux[0];
    vecteur[1] -= aux[1];
    vecteur[2] -= aux[2];    
  }
}
